unit Main_U;
// This file is part of Sarah Dean's Code Co-op Shell Extension.
//
//  "Sarah Dean's Code Co-op Shell Extension" is free software; you can
// redistribute it and/or modify it under the terms of the GNU General
// Public License as published by the Free Software Foundation; either
// version 2 of the License, or at your option) any later version.
//
//  "Sarah Dean's Code Co-op Shell Extension" is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with "Sarah Dean's Code Co-op Shell Extension"; if not, write to
// the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
// MA  02111-1307  USA



interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,
  SDCCIniFile_U, ExtCtrls;

type
  TForm2 = class(TForm)
    pbClose: TButton;
    pbAbout: TButton;
    GroupBox1: TGroupBox;
    ckSubMenu: TCheckBox;
    ckConfirmCheckouts: TCheckBox;
    Label1: TLabel;
    Label2: TLabel;
    Label5: TLabel;
    ckEnableShellExtn: TCheckBox;
    pbOK: TButton;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure pbSetClick(Sender: TObject);
    procedure pbCloseClick(Sender: TObject);
    procedure pbAboutClick(Sender: TObject);
  private
    SDCC_INI_FILE: TSDCCIniFile;

    // Attempt to determine if the shell extension is registered or not
    function IsShellExtnRegistered(): boolean;

    procedure RegisterShellSupport(enable: boolean);

  public
    { Public declarations }
  end;

var
  Form2: TForm2;

implementation

{$R *.DFM}

uses
  Registry,
  ComObj,  // Required for GUIDToString
  AppGlobalConsts_U,
  CodeCoop,
  About_U,
  FileCtrl;


procedure TForm2.FormShow(Sender: TObject);
var
  codecoopObj: TCodeCoop;
  installedOK: boolean;
begin
  self.Caption := APP_TITLE+': Config';


  ckEnableShellExtn.checked := IsShellExtnRegistered();

  ckSubMenu.checked := SDCC_INI_FILE.OptShellExtSubMenu;
  ckConfirmCheckouts.checked := SDCC_INI_FILE.OptShellExtConfirmCheckouts;

  codecoopObj:= TCodeCoop.Create();
  try
    codecoopObj.Active := TRUE;
    installedOK := codecoopObj.Active;

  finally
    codecoopObj.Free();
  end;

  if not(installedOK) then
    begin
    MessageDlg(
               'You do not appear to have installed the'+CRLF+
               'Code Co-op command line utilities.'+CRLF+
               CRLF+
               'Please install this package, and try again',
               mtError, [mbOK], 0);

    Close();
    Application.Terminate();
    end;

end;


procedure TForm2.FormCreate(Sender: TObject);
begin
  SDCC_INI_FILE := TSDCCIniFile.Create(INI_FILENAME);
  SDCC_INI_FILE.ReadIniFile();

end;

procedure TForm2.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  SDCC_INI_FILE.Free();

end;

procedure TForm2.pbSetClick(Sender: TObject);
begin
  SDCC_INI_FILE.OptShellExtSubMenu := ckSubMenu.checked;
  SDCC_INI_FILE.OptShellExtConfirmCheckouts := ckConfirmCheckouts.checked;
  SDCC_INI_FILE.WriteIniFile();

  RegisterShellSupport(ckEnableShellExtn.checked);

  Close();

end;

procedure TForm2.pbCloseClick(Sender: TObject);
begin
  Close();

end;


procedure TForm2.RegisterShellSupport(enable: boolean);
var
  Handle:  THandle;
//  RegProc: function(r: boolean): HResult; stdcall;
  RegProc: function(): HResult; stdcall;
  hr: HResult;
  callProc: string;
begin
  if enable then
    begin
    callProc := 'DllRegisterServer';
    end
  else
    begin
    callProc := 'DllUnregisterServer';
    end;

  // Call Register on the DLL with the value of reg
  Handle := LoadLibrary(SHELL_EXT_FILENAME);
  if Handle=0 then
    begin
    raise Exception.CreateFmt('%s: %s',
                              [SysErrorMessage(GetLastError),
                               SHELL_EXT_FILENAME]);
    end;

  try
    RegProc := GetProcAddress(Handle, PChar(callProc));
    if Assigned(RegProc) then
      begin
      hr := RegProc();
      if Failed(hr) then
        begin
        raise Exception.Create(callProc+' in '+SHELL_EXT_FILENAME+' failed.')
        end;
      end
    else
      begin
      RaiseLastWin32Error;
      end;
  finally
    FreeLibrary(Handle)
  end

end;


procedure TForm2.pbAboutClick(Sender: TObject);
var
  aboutDlg: TAbout_F;
begin
  aboutDlg:= TAbout_F.Create(nil);
  try
    aboutDlg.ShowModal();
  finally
    aboutDlg.Free();
  end;

end;


// Attempt to determine if the shell extension is registered or not
function TForm2.IsShellExtnRegistered(): boolean;
var
  ClassID: string;
  registry: TRegistry;
  retVal: boolean;
begin
  retVal:= FALSE;

  registry := TRegistry.create();
  try
    registry.RootKey := HKEY_CLASSES_ROOT;
    try
      ClassID := GUIDToString(Class_ContextMenu);

      // All extensions
      retVal := (
                 registry.KeyExists(REGKEY_ALLEXT) and
                 registry.KeyExists(REGKEY_DIR)
// Drive not needed
//                 registry.KeyExists(REGKEY_DRIVE)
                );

    except
      on ERegistryException do
        begin
        MessageDlg(
                   'An error occured while writing to the registry; if you are running under'+CRLF+
                   'NT, please ensure that you have the necessary permissions do perform'+CRLF+
                   'this operation (i.e. logon as administrator) and try again',
                   mtError,
                   [mbOK],
                   0
                  );
        end;
    end;

  finally
    registry.Free();
  end;

  Result := retVal;
end;




END.


