unit frmProgress_U;
// This file is part of Sarah Dean's Code Co-op Shell Extension.
//
//  "Sarah Dean's Code Co-op Shell Extension" is free software; you can
// redistribute it and/or modify it under the terms of the GNU General
// Public License as published by the Free Software Foundation; either
// version 2 of the License, or at your option) any later version.
//
//  "Sarah Dean's Code Co-op Shell Extension" is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with "Sarah Dean's Code Co-op Shell Extension"; if not, write to
// the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
// MA  02111-1307  USA


// Description: Progress bar dialog
// By Sarah Dean
//
// -----------------------------------------------------------------------------
//

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls,
  DirLabel;

type
  TfrmProgress = class(TForm)
    pgbOverall: TProgressBar;
    pbCancel: TButton;
    Label1: TLabel;
    lblDisplayCount: TLabel;
    DirLabel1: TDirLabel;
    procedure pbCancelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    fConfirmCancel: boolean;
    i64MinValue: int64;
    i64MaxValue: int64;
    i64PositionValue: int64;

    procedure UpdateI64ProgressBar();

    procedure iSetMax(max: integer);
    procedure iSetMin(min: integer);
    function  iGetMax(): integer;
    function  iGetMin(): integer;
    procedure iSetPosition(position: integer);
    procedure iSetInversePosition(position: integer);

    procedure i64SetMax(max: int64);
    procedure i64SetMin(min: int64);
    procedure i64SetPosition(position: int64);
    procedure i64SetInversePosition(position: int64);

    procedure UpdateDisplayCount();

  public
    Cancel: boolean;
    property  Max: integer read iGetMax write iSetMax;
    property  Min: integer read iGetMin write iSetMin;
    property  Position: integer write iSetPosition;
    property  InversePosition: integer write iSetInversePosition;
    procedure IncPosition(); overload;
    procedure IncPosition(x: integer); overload;

    property  i64Max: int64 write i64SetMax;
    property  i64Min: int64 write i64SetMin;
    property  i64Position: int64 write i64SetPosition;
    property  i64InversePosition: int64 write i64SetInversePosition;
    procedure i64IncPosition();

    property  ConfirmCancel: boolean write fConfirmCancel default TRUE;
    procedure Filename(filename: string);
  end;


implementation

{$R *.DFM}

uses SDUGeneral,
     math;

procedure TfrmProgress.UpdateI64ProgressBar();
var
  barLength: int64;
  barPos: int64;
  percentPos: integer;
begin
  iSetMin(0);
  iSetMax(100);

  barLength := i64MaxValue - i64MinValue;
  barPos := i64PositionValue - i64MinValue;

  if (i64MaxValue<>0) then
    begin
    percentPos := ((barPos*100) div barLength);
    iSetPosition(percentPos);
    end;

end;



procedure TfrmProgress.iSetMax(max: integer);
begin
  pgbOverall.Max := max;
  UpdateDisplayCount();
end;

procedure TfrmProgress.iSetMin(min: integer);
begin
  pgbOverall.Min := min;
  UpdateDisplayCount();
end;



function TfrmProgress.iGetMax(): integer;
begin
  Result := pgbOverall.Max;
end;

function TfrmProgress.iGetMin(): integer;
begin
  Result := pgbOverall.Min;
end;



procedure TfrmProgress.iSetPosition(position: integer);
begin
  if position>pgbOverall.Max then
    begin
    pgbOverall.position := pgbOverall.Max
    end
  else if position<pgbOverall.Min then
    begin
    pgbOverall.position := pgbOverall.Min
    end
  else
    begin
    pgbOverall.position := position;
    end;

  UpdateDisplayCount();

  self.refresh;
  Application.ProcessMessages();
end;

procedure TfrmProgress.iSetInversePosition(position: integer);
begin
  iSetPosition(pgbOverall.Max-position);

end;

procedure TfrmProgress.IncPosition();
begin
  IncPosition(1);

end;

procedure TfrmProgress.IncPosition(x: integer);
begin
  iSetPosition(pgbOverall.Position+x);

end;


procedure TfrmProgress.i64SetMax(max: int64);
begin
  i64MaxValue := max;
  UpdateI64ProgressBar();

end;

procedure TfrmProgress.i64SetMin(min: int64);
begin
  i64MinValue := min;
  UpdateI64ProgressBar();

end;

procedure TfrmProgress.i64SetPosition(position: int64);
begin
  i64PositionValue := position;
  UpdateI64ProgressBar();

end;

procedure TfrmProgress.i64SetInversePosition(position: int64);
begin
  i64PositionValue := i64MaxValue-position;
  UpdateI64ProgressBar();

end;

procedure TfrmProgress.i64IncPosition();
begin
  i64PositionValue := i64PositionValue + 1;
  UpdateI64ProgressBar();

end;


procedure TfrmProgress.pbCancelClick(Sender: TObject);
begin
  Cancel := TRUE;
  pbCancel.Enabled := FALSE;
  if fConfirmCancel then
    begin
    MessageDlg('Operation canceled by user', mtWarning, [mbOK], 0);
    end;

end;

procedure TfrmProgress.FormCreate(Sender: TObject);
begin
  pbCancel.Enabled := TRUE;
  Cancel := FALSE;

end;

procedure TfrmProgress.Filename(filename: string);
begin
  DirLabel1.DirName := filename;

end;

procedure TfrmProgress.FormShow(Sender: TObject);
begin
  Filename('');

end;

procedure TfrmProgress.UpdateDisplayCount();
begin
  lblDisplayCount.Caption := inttostr(pgbOverall.position)+'/'+inttostr(pgbOverall.Max);
end;

END.

